<?php

namespace Nest\Database\Query\Join;

/**
 * Add join capacity to a query builder.
 */
trait HasJoin
{
	/**
	 * Join clauses.
	 * @var JoinBuilder[]
	 */
	protected array $joins = [];

	/**
	 * Reset JOIN clauses.
	 * @return $this
	 */
	public function resetJoins(): static
	{
		$this->joins = [];
		return $this;
	}

	/**
	 * Create a new join clause.
	 * @param string $type Type of the join clause.
	 * @param string $table Joined table.
	 * @param string|null $as Joined table alias.
	 * @return JoinBuilder New join clause builder.
	 */
	public function join(string $type, string $table, ?string $as = null): JoinBuilder
	{
		$join = new JoinBuilder($this, $type, $table, $as);
		$this->joins[] = $join;
		return $join;
	}

	/**
	 * Create a new inner join clause.
	 * @param string $table Joined table.
	 * @param string|null $as Joined table alias.
	 * @return JoinBuilder New join clause builder.
	 */
	public function innerJoin(string $table, ?string $as = null): JoinBuilder
	{
		return $this->join(JoinBuilder::INNER, $table, $as);
	}

	/**
	 * Create a new outer join clause.
	 * @param string $table Joined table.
	 * @param string|null $as Joined table alias.
	 * @return JoinBuilder New join clause builder.
	 */
	public function outerJoin(string $table, ?string $as = null): JoinBuilder
	{
		return $this->join(JoinBuilder::OUTER, $table, $as);
	}

	/**
	 * Create a new left join clause.
	 * @param string $table Joined table.
	 * @param string|null $as Joined table alias.
	 * @return JoinBuilder New join clause builder.
	 */
	public function leftJoin(string $table, ?string $as = null): JoinBuilder
	{
		return $this->join(JoinBuilder::LEFT, $table, $as);
	}

	/**
	 * Create a new right join clause.
	 * @param string $table Joined table.
	 * @param string|null $as Joined table alias.
	 * @return JoinBuilder New join clause builder.
	 */
	public function rightJoin(string $table, ?string $as = null): JoinBuilder
	{
		return $this->join(JoinBuilder::RIGHT, $table, $as);
	}
}
